/******************************************************************************/
#include "mtc1type.h"
#include "mtc1ext.h"
/******************************************************************************/
/*                     Do processing for decoder 1.
*******************************************************************************/
int Dec1_Process()
{
 Init_Alpha_Dec1();
 Init_Beta_Dec1();
 Compute_Alpha_Dec1();
 Compute_Beta_Dec1();
// Print_Alpha_Beta();
 if(Iter < (MAX_ITER-ONE))
 {
  Compute_APPLes_Dec1();
  Compute_New_Le_Dec1();
 }
 if(Iter == (MAX_ITER-ONE))
  Compute_APPs_Dec1();
 return(0);
}
/******************************************************************************/
/*                 Initialize alpha[0][state] for 1st decoder.
*******************************************************************************/
int Init_Alpha_Dec1()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Get_Init_Alpha01(state);
 return(0);
}
/******************************************************************************/
/* Get the initial value of alpha[0][state] for a given state for 1st decoder.
*******************************************************************************/
int Get_Init_Alpha01(state)
int state;
{
 int input,prv_st,op;
 double ge,gi,prob;
 Alpha[ZERO][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=Trel_PS[state][input];
  op=Trel_Op[prv_st][input];
  ge=Gamma_E_P1[ZERO][op];                          /* Corresponds to parity. */
  gi=Gamma_I[ZERO][input];                            /* Corresponds to data. */
  if(input == ZERO)
   prob=Le[ZERO]/(1.0+Le[ZERO]);
  else
   prob=1.0/(1.0+Le[ZERO]);
  Alpha[ZERO][state]=Alpha[ZERO][state]+ge*gi*prob;
 }
 return(0);
}
/******************************************************************************/
/*       Initialize the value of beta[FRAME_SIZE-1][state] for decoder 1.
*******************************************************************************/
int Init_Beta_Dec1()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Get_Init_Beta_Dec1(state);
 return(0);
}
/******************************************************************************/
/*            Get initial beta for decoder 1 for a particular state.
*******************************************************************************/
int Get_Init_Beta_Dec1(state)
int state;
{
 int input,op;
 double ge,gi,prob;
 Beta[FRAME_SIZE-ONE][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  op=Trel_Op[state][input];                               /* Get encoded bit. */
  ge=Gamma_E_P1[FRAME_SIZE-ONE][op];                /* Corresponds to parity. */
  gi=Gamma_I[FRAME_SIZE-ONE][input];                  /* Corresponds to data. */
  if(input == ZERO)
   prob=Le[FRAME_SIZE-ONE]/(1.0+Le[FRAME_SIZE-ONE]);
  else
   prob=1.0/(1.0+Le[FRAME_SIZE-ONE]);
  Beta[FRAME_SIZE-ONE][state]=Beta[FRAME_SIZE-ONE][state]+ge*gi*prob;
 }
 return(0);
}
/******************************************************************************/
/*                        Compute the alpha values.
*******************************************************************************/
int Compute_Alpha_Dec1()
{
 int time,state;
 for(time=ONE;time<FRAME_SIZE-ONE;time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Alpha_Dec1(time,state);
  Normalize_Alpha(time);
 }
 return(0);
}
/******************************************************************************/
/*           Compute the new alpha value for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Dec1(time,state)
int time,state;
{
 int input,prv_st,op;
 double ge,gi,prob,prv_alpha;
 Alpha[time][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Trel_PS[state][input];
  op=Trel_Op[prv_st][input];                              /* Get encoded bit. */
  ge=Gamma_E_P1[time][op];                          /* Corresponds to parity. */
  gi=Gamma_I[time][input];                            /* Corresponds to data. */
  if(input == ZERO)
   prob=Le[time]/(1.0+Le[time]);
  else
   prob=1.0/(1.0+Le[time]);
  prv_alpha=Alpha[time-ONE][prv_st];
  Alpha[time][state]=Alpha[time][state]+ge*gi*prob*prv_alpha;
 }
 return(0);
}
/******************************************************************************/
/*                        Compute the beta values.
*******************************************************************************/
int Compute_Beta_Dec1()
{
 int time,state;
 for(time=FRAME_SIZE-2;time>ZERO;time--)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Beta_Dec1(time,state);
  Normalize_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*           Compute the new beta value for a given state and time.
*******************************************************************************/
int Get_New_Beta_Dec1(time,state)
int time,state;
{
 int input,nxt_st,op;
 double ge,gi,prob,nxt_beta;
 Beta[time][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  nxt_st=Trel_NS[state][input];
  op=Trel_Op[state][input];                               /* Get encoded bit. */
  ge=Gamma_E_P1[time][op];                          /* Corresponds to parity. */
  gi=Gamma_I[time][input];                            /* Corresponds to data. */
  if(input == ZERO)
   prob=Le[time]/(1.0+Le[time]);
  else
   prob=1.0/(1.0+Le[time]);
  nxt_beta=Beta[time+ONE][nxt_st];
  Beta[time][state]=Beta[time][state]+ge*gi*prob*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec1()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Due_to_P1_Dec1(time);
  Get_APPLe_Due_to_M1_Dec1(time);
 }
 return(0);
}
/******************************************************************************/
/*    Get a posteriori probability due to +1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Due_to_P1_Dec1(time)
int time;
{
 int state,nxt_state,op;
 double ge,nxt_beta,prv_alpha;
 App_P1[time]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  nxt_state=Trel_NS[state][ZERO];                            /* 0 maps to +1. */
  op=Trel_Op[state][ZERO];
  ge=Gamma_E_P1[time][op];                          /* Corresponds to parity. */
  prv_alpha=nxt_beta=1.0;
  if(time > ZERO)
   prv_alpha=Alpha[time-ONE][state];
  if(time < (FRAME_SIZE-ONE))
   nxt_beta=Beta[time+ONE][nxt_state];
  App_P1[time]=App_P1[time]+ge*prv_alpha*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*    Get a posteriori probability due to -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Due_to_M1_Dec1(time)
int time;
{
 int state,nxt_state,op;
 double ge,nxt_beta,prv_alpha;
 App_M1[time]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  nxt_state=Trel_NS[state][ONE];                             /* 1 maps to -1. */
  op=Trel_Op[state][ONE];
  ge=Gamma_E_P1[time][op];                          /* Corresponds to parity. */
  prv_alpha=nxt_beta=1.0;
  if(time > ZERO)
   prv_alpha=Alpha[time-ONE][state];
  if(time < (FRAME_SIZE-ONE))
   nxt_beta=Beta[time+ONE][nxt_state];
  App_M1[time]=App_M1[time]+ge*prv_alpha*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*                    Compute new Le due to decoder 1.
*******************************************************************************/
Compute_New_Le_Dec1()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(App_P1[time] > (100000.0*App_M1[time]))
   Le[time]=100000.0;
  else
   Le[time]=App_P1[time]/App_M1[time];
//  printf("dec1 iter:%d ap1:%lf am1:%lf\n",Iter,App_P1[time],App_M1[time]);
 }
 return(0);
}
/******************************************************************************/
