/******************************************************************************/
#include "mtc1type.h"
#include "mtc1ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Add_Noise();
  Receive();
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<FRAME_SIZE;cnt++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  N2_Val[URADIUS]=erand48(N3_Store);
  N2_Val[UTHETA]=erand48(N4_Store);
  My_Gauss(VARIANCE,N1_Val);
  My_Gauss(VARIANCE,N2_Val);
  Sym0[cnt]=Sym0[cnt]+N1_Val[GRAND_0];
  Sym1[cnt]=Sym1[cnt]+N1_Val[GRAND_1];
  Sym2[cnt]=Sym2[cnt]+N2_Val[GRAND_0];
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Gamma_I_E();
 Init_Le();
 Bcjr();
 Get_Bit_Errors_Dec1();
 return(0);
}
/******************************************************************************/
/*          Initialize the intrinsic and extrinsic values of gamma.
*******************************************************************************/
int Init_Gamma_I_E()
{
 Init_Gamma_I_E_Dec1();
 Init_Gamma_I_E_Dec2();
 return(0);
}
/******************************************************************************/
/*    Initialize the intrinsic and extrinsic values of gamma for decoder 1.
*******************************************************************************/
int Init_Gamma_I_E_Dec1()
{
 int time,bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
  {
   Compute_Gamma_E_P1(bit,time);
   Compute_Gamma_I(bit,time);
  }
 }
 return(0);
}
/******************************************************************************/
/* Compute gamma from parity bit (extrinsic information) for decoder 1 for
   odd time instants. This is to simulate puncturing.
*******************************************************************************/
int Compute_Gamma_E_P1(bit,time)
int bit,time;
{
 double sym,x;
 if((time%2) == ONE)
 {
  sym=Bit_Map[bit];
  x=sym-Sym1[time];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P1[time][bit]=exp(x);
 }
 else
  Gamma_E_P1[time][bit]=1.0;
// printf("time:%d bit:%d x:%lf ge1:%lf\n",time,bit,x,Gamma_E_P1[time][bit]);
 return(0);
}
/******************************************************************************/
/*      Compute gamma from data bit (intrinsic information) for decoder 1.
*******************************************************************************/
int Compute_Gamma_I(bit,time)
int bit,time;
{
 double sym,x;
 sym=Bit_Map[bit];
 x=sym-Sym0[time];
 x= -x*x*Inv_Twice_Noise_Var;
 Gamma_I[time][bit]=exp(x);
// printf("time:%d bit:%d x:%lf gi:%lf\n",time,bit,x,Gamma_I[time][bit]);
 return(0);
}
/******************************************************************************/
/*    Initialize the intrinsic and extrinsic values of gamma for decoder 2.
*******************************************************************************/
int Init_Gamma_I_E_Dec2()
{
 int time,bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(bit=ZERO;bit<NUM_INPUTS;bit++)
   Compute_Gamma_E_P2(bit,time);
 }
 return(0);
}
/******************************************************************************/
/* Compute gamma from parity bit (extrinsic information) for decoder 2 for
   even time instants. This is to simulate puncturing.
*******************************************************************************/
int Compute_Gamma_E_P2(bit,time)
int bit,time;
{
 double sym,x;
 if((time%2) == ZERO)
 {
  sym=Bit_Map[bit];
  x=sym-Sym2[time];
  x= -x*x*Inv_Twice_Noise_Var;
  Gamma_E_P2[time][bit]=exp(x);
 }
 else
  Gamma_E_P2[time][bit]=1.0;
 return(0);
}
/******************************************************************************/
/*                Initialize the ratio of the probabilities.
*******************************************************************************/
int Init_Le()
{
 int cnt;
 for(cnt=ZERO;cnt<FRAME_SIZE;cnt++)
  Le[cnt]=1.0;
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec1_Process();
  if(Iter < (MAX_ITER-ONE))
   Dec2_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(App_P1[time] > App_M1[time])
   detected_bit=ZERO;
  else
   detected_bit=ONE;
  if(detected_bit != Bits[time])
   Ber++;
 }
 return(0);
}
/******************************************************************************/
