/******************************************************************************/
#include "tc1ctype.h"
#include "tc1cext.h"
/******************************************************************************/
/*                   Initialize simulation variables.
*******************************************************************************/
int Sim_Init()
{
 Noise_Gen_Init();
 Iv_Noise_Gen_Init();
 Coloured_Noise_Filter_Init();
 Pred_Filter_Init();
 Inv_Pred_Var_Init();
 Ber_Init();
 return(0);
}
/******************************************************************************/
/*                     Initialize the noise generator.
*******************************************************************************/
int Noise_Gen_Init()
{
 int cnt;
 long int random(void);
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
 {
  N1_Store[cnt]=(unsigned short)random();
  N2_Store[cnt]=(unsigned short)random();
  N3_Store[cnt]=(unsigned short)random();
  N4_Store[cnt]=(unsigned short)random();
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the interleaver noise generator.
*******************************************************************************/
int Iv_Noise_Gen_Init()
{
 Iv_Noise_Store[0]=15131;
 Iv_Noise_Store[1]=13111;
 Iv_Noise_Store[2]=10591;
 return(0);
}
/******************************************************************************/
/*          Initialize the filters that generate coloured noise.
*******************************************************************************/
int Coloured_Noise_Filter_Init()
{
 if(FILTER == 1)
  Filter1_Init();
 else if(FILTER == 2)
  Filter2_Init();
 return(0);
}
/******************************************************************************/
/*                        Initialize the first filter.
*******************************************************************************/
int Filter1_Init()
{
 double tmp;
 tmp=0.6;
 Ip_Coeff=sqrt(1.0-tmp*tmp);
 Coeff_Array[0]= -tmp;
 return(0);
}
/******************************************************************************/
/*                        Initialize the second filter.
*******************************************************************************/
int Filter2_Init()
{
 int cnt;
 FILE *fp;
 fp=fopen("tc1cdata/f2coeff.dat","w");
 Filter_Array[0]=1.0/sqrt(91.0);
 Filter_Array[1]=2.0/sqrt(91.0);
 Filter_Array[2]=3.0/sqrt(91.0);
 Filter_Array[3]=4.0/sqrt(91.0);
 Filter_Array[4]=5.0/sqrt(91.0);
 Filter_Array[5]=6.0/sqrt(91.0);
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
 {
  printf("\t Type 2 filter coeff:%d is :%lf\n",cnt,Filter_Array[cnt]);
  fprintf(fp,"%lf %lf\n",(double)cnt,Filter_Array[cnt]);
 }
 printf("\n");
 Get_Auto_Corr();
 fclose(fp);
 return(0);
}
/******************************************************************************/
/*                Get the autocorrelation of the type 2 filter.
*******************************************************************************/
int Get_Auto_Corr()
{
 int cnt,cor_inx;
 double mf[FILTER_SIZE],op[2*FILTER_SIZE-ONE];
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
  mf[cnt]=Filter_Array[FILTER_SIZE-ONE-cnt];
 Convolve(Filter_Array,mf,op,FILTER_SIZE,FILTER_SIZE);
 for(cnt=FILTER_SIZE-ONE;cnt<2*FILTER_SIZE-ONE;cnt++)
 {
  cor_inx=cnt-FILTER_SIZE+ONE;
  Auto_Cor[cnt-FILTER_SIZE+ONE]=VARIANCE*op[cnt];
  printf("\t Index:%d Autocor:%lf\n",cor_inx,Auto_Cor[cor_inx]);
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                       Initialize the prediction filter.
*******************************************************************************/
int Pred_Filter_Init()
{
 double denom;
 if((FILTER == 2) && (PRED_MEM_SIZE == 1))
  Coeff_Array[0]= -Auto_Cor[ONE]/Auto_Cor[ZERO];
 else if((FILTER == 2) && (PRED_MEM_SIZE == 2))
 {
  denom=Auto_Cor[ZERO]*Auto_Cor[ZERO]-Auto_Cor[ONE]*Auto_Cor[ONE];
  Coeff_Array[0]=Auto_Cor[ONE]*(Auto_Cor[TWO]-Auto_Cor[ZERO])/denom;
  Coeff_Array[1]=(Auto_Cor[1]*Auto_Cor[1]-Auto_Cor[2]*Auto_Cor[0])/denom;
 }
 return(0);
}
/******************************************************************************/
/*          Initialize the inverse of the prediction error variance.
*******************************************************************************/
int Inv_Pred_Var_Init()
{
 int cnt;
 double err_var;
 if(FILTER == 1)
  Inv_Twice_Pred_Var=1.0/(2.0*VARIANCE*Ip_Coeff*Ip_Coeff);
 else if(FILTER == 2)
 {
  err_var=Auto_Cor[ZERO];
  for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
   err_var=err_var+Coeff_Array[cnt]*Auto_Cor[cnt+ONE];
  Inv_Twice_Pred_Var=1.0/(2.0*err_var);
 }
 printf("\t Inv. of twice the pred. error var:%lf\n\n",Inv_Twice_Pred_Var);
 return(0);
}
/******************************************************************************/
/*                       Initialize the bit error rate.
*******************************************************************************/
int Ber_Init()
{
 Ber=ZERO;
 SS_Detect_Error=ZERO;
 return(0);
}
/******************************************************************************/
