/******************************************************************************/
#include "tc1ctype.h"
#include "tc1cext.h"
/******************************************************************************/
/*                             Get the super trellis.
*******************************************************************************/
int Get_Super_Trellis()
{
 int st_cnt,mod_ip_cnt,ip_array[PRED_MEM_SIZE],parity[PRED_MEM_SIZE],new_st;
 for(st_cnt=ZERO;st_cnt<NUM_STATES;st_cnt++)
 {
  for(mod_ip_cnt=ZERO;mod_ip_cnt<MOD_NUM_INPUTS;mod_ip_cnt++)
  {
   Decimal_to_Binary(mod_ip_cnt,ip_array,PRED_MEM_SIZE);
   new_st=Fill_Parity_Pred_Mem(st_cnt,mod_ip_cnt,ip_array,parity);
   Get_G_I_G_E_Indices(new_st,mod_ip_cnt,ip_array,parity);
   Get_New_Super_State(new_st,mod_ip_cnt,ip_array);
  }
 }
 Get_Prv_States();
 Print_Super_Trellis();
 return(0);
}
/******************************************************************************/
/* Fill the predictor memory with the parity bits. Most recent input bit is
 * at ZERO. Most recent parity is also at ZERO.
*******************************************************************************/
int Fill_Parity_Pred_Mem(st_cnt,mod_ip_cnt,ip_array,parity)
int st_cnt,mod_ip_cnt,ip_array[],parity[];
{
 int cnt,inp,s_state;
 printf("\t Enc starting state:%d\n",st_cnt);
 for(cnt=PRED_MEM_SIZE-ONE;cnt>=ZERO;cnt--)
 {
  inp=ip_array[cnt];
  parity[cnt]=Trel_Op[st_cnt][inp];
  printf("\t Input:%d ",inp);
  printf("Parity:%d\n",parity[cnt]);
  st_cnt=Trel_NS[st_cnt][inp];
 }
 s_state=st_cnt*MOD_NUM_INPUTS+mod_ip_cnt;
 printf("\t Final encoder state:%d\n",st_cnt);
 printf("\t Final super trellis state:%d\n\n",s_state);
 return(st_cnt);
}
/******************************************************************************/
/* Fill the predictor memory with the indices corresponding to the input bits
 * and parity bits. Most recent bit is at ZERO.
*******************************************************************************/
int Get_G_I_G_E_Indices(new_st,mod_ip_cnt,ip_array,parity)
int new_st,mod_ip_cnt,ip_array[],parity[];
{
 int ip_cnt,s_state,cnt,t_ip[PRED_MEM_SIZE+1],t_parity[PRED_MEM_SIZE+1];
 s_state=new_st*MOD_NUM_INPUTS+mod_ip_cnt;
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
 {
  t_ip[cnt+ONE]=ip_array[cnt];
  t_parity[cnt+ONE]=parity[cnt];
 }
 for(ip_cnt=ZERO;ip_cnt<NUM_INPUTS;ip_cnt++)
 {
  t_ip[ZERO]=ip_cnt;
  t_parity[ZERO]=Trel_Op[new_st][ip_cnt];
  STrel_G_I[s_state][ip_cnt]=Binary_to_Decimal(t_ip,PRED_MEM_SIZE+1);
  STrel_G_E[s_state][ip_cnt]=Binary_to_Decimal(t_parity,PRED_MEM_SIZE+1);
 }
 return(0);
}
/******************************************************************************/
/*                         Get the new super state.
*******************************************************************************/
int Get_New_Super_State(new_st,mod_ip_cnt,ip_array)
int new_st,mod_ip_cnt,ip_array[];
{
 int super_state,cnt,inx,enc_nxt_state,enc_op,pred_nxt_state;
 int temp[PRED_MEM_SIZE],new_super_state;
 super_state=new_st*MOD_NUM_INPUTS+mod_ip_cnt;
 for(cnt=ZERO;cnt<NUM_INPUTS;cnt++)
 {
  enc_nxt_state=Trel_NS[new_st][cnt];
  enc_op=Trel_Op[new_st][cnt];
  for(inx=ZERO;inx<PRED_MEM_SIZE-ONE;inx++)
   temp[inx+ONE]=ip_array[inx];
  temp[ZERO]=cnt;
  pred_nxt_state=Binary_to_Decimal(temp,PRED_MEM_SIZE);
  new_super_state=enc_nxt_state*MOD_NUM_INPUTS+pred_nxt_state;
  STrel_NS[super_state][cnt]=new_super_state;
  STrel_Op[super_state][cnt]=enc_op;
 }
 return(0);
}
/******************************************************************************/
/*               Get the previous states in the super trellis.
*******************************************************************************/
int Get_Prv_States()
{
 int st_cnt,ip_cnt,nxt_state,inx,index[NUM_ST_STATES];
 Init_Index(index);
 for(st_cnt=ZERO;st_cnt<NUM_ST_STATES;st_cnt++)
 {
  for(ip_cnt=ZERO;ip_cnt<NUM_INPUTS;ip_cnt++)
  {
   nxt_state=STrel_NS[st_cnt][ip_cnt];
   inx=index[nxt_state];
   if(inx >= NUM_INPUTS)
    Error_Mes(8);
   STrel_PS[nxt_state][inx]=st_cnt;
   STrel_Ip[nxt_state][inx]=ip_cnt;
   index[nxt_state]++;
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Initialize the index array.
*******************************************************************************/
int Init_Index(index)
int index[];
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_ST_STATES;cnt++)
  index[cnt]=ZERO;
 return(0);
}
/******************************************************************************/
/*                          Print the super trellis.
*******************************************************************************/
int Print_Super_Trellis()
{
 int st_cnt,ip_cnt,inx;
 printf("\t Super trellis:\n\n");
 for(st_cnt=ZERO;st_cnt<NUM_ST_STATES;st_cnt++)
 {
  for(ip_cnt=ZERO;ip_cnt<NUM_INPUTS;ip_cnt++)
  {
   printf("\t Cur. st.:%4d Inp:%2d ",st_cnt,ip_cnt);
   printf("Nxt st.:%4d ",STrel_NS[st_cnt][ip_cnt]);
   printf("Prev st.:%4d ",STrel_PS[st_cnt][ip_cnt]);
   printf("Prev ip:%4d ",STrel_Ip[st_cnt][ip_cnt]);
   printf("Op:%2d\n",STrel_Op[st_cnt][ip_cnt]);
   printf("\t Intrinsic index:%3d ",STrel_G_I[st_cnt][ip_cnt]);
   printf("Extrinsic index:%3d\n\n",STrel_G_E[st_cnt][ip_cnt]);
  }
 }
 return(0);
}
/******************************************************************************/
