/******************************************************************************/
#include "tc2type.h"
#include "tc2ext.h"
/******************************************************************************/
/*                     Do processing for decoder 2.
*******************************************************************************/
int Dec2_Process()
{
 Init_Alpha_Dec2();
 Init_Beta_Dec2();
 Compute_Alpha_Dec2();
 Compute_Beta_Dec2();
 Compute_APPLes_Dec2();
 Compute_New_Le_Dec2();
 Compute_Dv_Le_Dec2();
// Compute_APPs_Dec2();
 return(0);
}
/******************************************************************************/
/*                 Initialize alpha[0][state] for 2nd decoder.
*******************************************************************************/
int Init_Alpha_Dec2()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Get_Init_Alpha02(state);
 return(0);
}
/******************************************************************************/
/* Get the initial value of alpha[0][state] for a given state for 2nd decoder.
*******************************************************************************/
int Get_Init_Alpha02(state)
int state;
{
 int input,prv_st,op;
 double ge,gi,prob;
 Alpha[ZERO][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)
 {
  prv_st=Trel_PS[state][input];
  op=Trel_Op[prv_st][input];
  ge=Gamma_E_P2[ZERO][op];                          /* Corresponds to parity. */
  gi=Gamma_Iv_I[ZERO][input];                         /* Corresponds to data. */
  if(input == ZERO)
   prob=Iv_Le[ZERO]/(1.0+Iv_Le[ZERO]);
  else
   prob=1.0/(1.0+Iv_Le[ZERO]);
  Alpha[ZERO][state]=Alpha[ZERO][state]+ge*gi*prob;
 }
 return(0);
}
/******************************************************************************/
/*       Initialize the value of beta[FRAME_SIZE-1][state] for decoder 2.
*******************************************************************************/
int Init_Beta_Dec2()
{
 int state;
 for(state=ZERO;state<NUM_STATES;state++)
  Get_Init_Beta_Dec2(state);
 return(0);
}
/******************************************************************************/
/*            Get initial beta for decoder 2 for a particular state.
*******************************************************************************/
int Get_Init_Beta_Dec2(state)
int state;
{
 int input,op;
 double ge,gi,prob;
 Beta[FRAME_SIZE-ONE][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  op=Trel_Op[state][input];                               /* Get encoded bit. */
  ge=Gamma_E_P2[FRAME_SIZE-ONE][op];                /* Corresponds to parity. */
  gi=Gamma_Iv_I[FRAME_SIZE-ONE][input];               /* Corresponds to data. */
  if(input == ZERO)
   prob=Iv_Le[FRAME_SIZE-ONE]/(1.0+Iv_Le[FRAME_SIZE-ONE]);
  else
   prob=1.0/(1.0+Iv_Le[FRAME_SIZE-ONE]);
  Beta[FRAME_SIZE-ONE][state]=Beta[FRAME_SIZE-ONE][state]+ge*gi*prob;
 }
 return(0);
}
/******************************************************************************/
/*                        Compute the alpha values.
*******************************************************************************/
int Compute_Alpha_Dec2()
{
 int time,state;
 for(time=ONE;time<(FRAME_SIZE-ONE);time++)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Alpha_Dec2(time,state);
  Normalize_Alpha(time);
 }
 return(0);
}
/******************************************************************************/
/*           Compute the new alpha value for a given state and time.
*******************************************************************************/
int Get_New_Alpha_Dec2(time,state)
int time,state;
{
 int input,prv_st,op;
 double ge,gi,prob,prv_alpha;
 Alpha[time][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  prv_st=Trel_PS[state][input];
  op=Trel_Op[prv_st][input];                              /* Get encoded bit. */
  ge=Gamma_E_P2[time][op];                          /* Corresponds to parity. */
  gi=Gamma_Iv_I[time][input];                         /* Corresponds to data. */
  if(input == ZERO)
   prob=Iv_Le[time]/(1.0+Iv_Le[time]);
  else
   prob=1.0/(1.0+Iv_Le[time]);
  prv_alpha=Alpha[time-ONE][prv_st];
  Alpha[time][state]=Alpha[time][state]+ge*gi*prob*prv_alpha;
 }
 return(0);
}
/******************************************************************************/
/*                        Compute the beta values.
*******************************************************************************/
int Compute_Beta_Dec2()
{
 int time,state;
 for(time=FRAME_SIZE-2;time>ZERO;time--)
 {
  for(state=ZERO;state<NUM_STATES;state++)
   Get_New_Beta_Dec2(time,state);
  Normalize_Beta(time);
 }
 return(0);
}
/******************************************************************************/
/*           Compute the new beta value for a given state and time.
*******************************************************************************/
int Get_New_Beta_Dec2(time,state)
int time,state;
{
 int input,nxt_st,op;
 double ge,gi,prob,nxt_beta;
 Beta[time][state]=0.0;
 for(input=ZERO;input<NUM_INPUTS;input++)                       /* Data bits. */
 {
  nxt_st=Trel_NS[state][input];
  op=Trel_Op[state][input];                               /* Get encoded bit. */
  ge=Gamma_E_P2[time][op];                          /* Corresponds to parity. */
  gi=Gamma_Iv_I[time][input];                         /* Corresponds to data. */
  if(input == ZERO)
   prob=Iv_Le[time]/(1.0+Iv_Le[time]);
  else
   prob=1.0/(1.0+Iv_Le[time]);
  nxt_beta=Beta[time+ONE][nxt_st];
  Beta[time][state]=Beta[time][state]+ge*gi*prob*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*           Compute the APPs due to extrinsic information alone.
*******************************************************************************/
int Compute_APPLes_Dec2()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APPLe_Due_to_P1_Dec2(time);
  Get_APPLe_Due_to_M1_Dec2(time);
 }
 return(0);
}
/******************************************************************************/
/*    Get a posteriori probability due to +1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Due_to_P1_Dec2(time)
int time;
{
 int state,nxt_state,op;
 double ge,nxt_beta,prv_alpha;
 App_P1[time]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  nxt_state=Trel_NS[state][ZERO];                            /* 0 maps to +1. */
  op=Trel_Op[state][ZERO];
  ge=Gamma_E_P2[time][op];                          /* Corresponds to parity. */
  prv_alpha=nxt_beta=1.0;
  if(time > ZERO)
   prv_alpha=Alpha[time-ONE][state];
  if(time < (FRAME_SIZE-ONE))
   nxt_beta=Beta[time+ONE][nxt_state];
  App_P1[time]=App_P1[time]+ge*prv_alpha*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*    Get a posteriori probability due to -1 due to extrinsic info alone.
*******************************************************************************/
int Get_APPLe_Due_to_M1_Dec2(time)
int time;
{
 int state,nxt_state,op;
 double ge,nxt_beta,prv_alpha;
 App_M1[time]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  nxt_state=Trel_NS[state][ONE];                             /* 1 maps to -1. */
  op=Trel_Op[state][ONE];
  ge=Gamma_E_P2[time][op];                          /* Corresponds to parity. */
  prv_alpha=nxt_beta=1.0;
  if(time > ZERO)
   prv_alpha=Alpha[time-ONE][state];
  if(time < (FRAME_SIZE-ONE))
   nxt_beta=Beta[time+ONE][nxt_state];
  App_M1[time]=App_M1[time]+ge*prv_alpha*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*                    Compute new Le due to decoder 2.
*******************************************************************************/
Compute_New_Le_Dec2()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
  Iv_Le[time]=App_P1[time]/App_M1[time];
 return(0);
}
/******************************************************************************/
/*                    Compute deinterleaved Le due to decoder 2.
*******************************************************************************/
int Compute_Dv_Le_Dec2()
{
 int time,iv_inx;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  iv_inx=Iv_Map[time];
  Dv_Le[time]=Iv_Le[iv_inx];
 }
 return(0);
}
/******************************************************************************/
/*                        Compute the APP ratios.
*******************************************************************************/
int Compute_APPs_Dec2()
{
 int time;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  Get_APP_Due_to_P1_Dec2(time);
  Get_APP_Due_to_M1_Dec2(time);
 }
 return(0);
}
/******************************************************************************/
/*                 Get a posteriori probability due to +1.
*******************************************************************************/
int Get_APP_Due_to_P1_Dec2(time)
int time;
{
 int state,nxt_state,op;
 double ge,gi,prob,nxt_beta,prv_alpha;
 App_P1[time]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  nxt_state=Trel_NS[state][ZERO];                            /* 0 maps to +1. */
  op=Trel_Op[state][ZERO];
  ge=Gamma_E_P2[time][op];                          /* Corresponds to parity. */
  gi=Gamma_Iv_I[time][ZERO];                          /* Corresponds to data. */
  prob=Iv_Le[time]/(1.0+Iv_Le[time]);
  prv_alpha=nxt_beta=1.0;
  if(time > ZERO)
   prv_alpha=Alpha[time-ONE][state];
  if(time < (FRAME_SIZE-ONE))
   nxt_beta=Beta[time+ONE][nxt_state];
  App_P1[time]=App_P1[time]+ge*gi*prob*prv_alpha*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
/*                 Get a posteriori probability due to -1.
*******************************************************************************/
int Get_APP_Due_to_M1_Dec2(time)
int time;
{
 int state,nxt_state,op;
 double ge,gi,prob,nxt_beta,prv_alpha;
 App_M1[time]=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  nxt_state=Trel_NS[state][ONE];                             /* 1 maps to -1. */
  op=Trel_Op[state][ONE];
  ge=Gamma_E_P2[time][op];                          /* Corresponds to parity. */
  gi=Gamma_Iv_I[time][ONE];                           /* Corresponds to data. */
  prob=1.0/(1.0+Iv_Le[time]);
  prv_alpha=nxt_beta=1.0;
  if(time > ZERO)
   prv_alpha=Alpha[time-ONE][state];
  if(time < (FRAME_SIZE-ONE))
   nxt_beta=Beta[time+ONE][nxt_state];
  App_M1[time]=App_M1[time]+ge*gi*prob*prv_alpha*nxt_beta;
 }
 return(0);
}
/******************************************************************************/
