/******************************************************************************/
#include "tc3ctype.h"
#include "tc3cext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  Add_Noise();
  if(Frame_Cnt > ZERO)
   Receive();
 }
 return(0);
}
/******************************************************************************/
/*                                Add noise.
*******************************************************************************/
int Add_Noise()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<FRAME_SIZE;cnt++)
 {
  N1_Val[URADIUS]=erand48(N1_Store);
  N1_Val[UTHETA]=erand48(N2_Store);
  N2_Val[URADIUS]=erand48(N3_Store);
  N2_Val[UTHETA]=erand48(N4_Store);
  My_Gauss(VARIANCE,N1_Val);
  My_Gauss(VARIANCE,N2_Val);
  if(WHITE)
   Add_White_Noise(cnt);
  else
   Add_Coloured_Noise(cnt);
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_White_Noise(time)
int time;
{
 Re_Sym0[time]=Re_Sym0[time]+N1_Val[GRAND_0];
 Im_Sym0[time]=Im_Sym0[time]+N1_Val[GRAND_1];
 Re_Sym1[time]=Re_Sym1[time]+N2_Val[GRAND_0];
 Im_Sym1[time]=Im_Sym1[time]+N2_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                     Get coloured noise from type 2 filter.
*******************************************************************************/
int Add_Coloured_Noise(time)
int time;
{
 int cnt;
 Re_Noise_0=Im_Noise_0=Re_Noise_1=Im_Noise_1=0.0;
 Shift_New_Noise_Into_Filter();
 for(cnt=ZERO;cnt<FILTER_SIZE;cnt++)
 {
  Re_Noise_0=Re_Noise_0+Re_F0_Reg[cnt]*Re_Flt[cnt];
  Re_Noise_0=Re_Noise_0-Im_F0_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_0=Im_Noise_0+Re_F0_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_0=Im_Noise_0+Im_F0_Reg[cnt]*Re_Flt[cnt];
  Re_Noise_1=Re_Noise_1+Re_F1_Reg[cnt]*Re_Flt[cnt];
  Re_Noise_1=Re_Noise_1-Im_F1_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_1=Im_Noise_1+Re_F1_Reg[cnt]*Im_Flt[cnt];
  Im_Noise_1=Im_Noise_1+Im_F1_Reg[cnt]*Re_Flt[cnt];
 }
 Re_Sym0[time]=Re_Sym0[time]+Re_Noise_0;
 Im_Sym0[time]=Im_Sym0[time]+Im_Noise_0;
 Re_Sym1[time]=Re_Sym1[time]+Re_Noise_1;
 Im_Sym1[time]=Im_Sym1[time]+Im_Noise_1;
 return(0);
}
/******************************************************************************/
/*              Shift in the new noise samples into type 2 filter.
*******************************************************************************/
int Shift_New_Noise_Into_Filter()
{
 int cnt;
 for(cnt=FILTER_SIZE-ONE;cnt>ZERO;cnt--)
 {
  Re_F0_Reg[cnt]=Re_F0_Reg[cnt-ONE];
  Im_F0_Reg[cnt]=Im_F0_Reg[cnt-ONE];
  Re_F1_Reg[cnt]=Re_F1_Reg[cnt-ONE];
  Im_F1_Reg[cnt]=Im_F1_Reg[cnt-ONE];
 }
 Re_F0_Reg[ZERO]=N1_Val[GRAND_0];
 Im_F0_Reg[ZERO]=N1_Val[GRAND_1];
 Re_F1_Reg[ZERO]=N2_Val[GRAND_0];
 Im_F1_Reg[ZERO]=N2_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Gamma();
 Init_App();
 Bcjr();
 Get_Bit_Errors_Dec1();
 return(0);
}
/******************************************************************************/
/* Initialize the intrinsic and extrinsic values of gamma for decoder 1. Most
 * recent bit is at location ZERO.
*******************************************************************************/
int Init_Gamma()
{
 int time,inx,sum,ip_dec,p_dec;
 int ip_array[PRED_MEM_SIZE+1],p_array[PRED_MEM_SIZE+1];
 for(inx=ZERO;inx<Inx_To_VG_Tbl;inx++)
 {
  sum=Valid_Gamma_Inx[inx];
  ip_dec=sum%MOD1_NUM_INPUTS;
  p_dec=sum/MOD1_NUM_INPUTS;
  Decimal_to_Binary(ip_dec,ip_array,PRED_MEM_SIZE+ONE);
  Decimal_to_Binary(p_dec,p_array,PRED_MEM_SIZE+ONE);
  for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
  {
   Compute_Gamma_Dec1(inx,time,ip_array,p_array);
   Compute_Gamma_Dec2(inx,time,ip_array,p_array);
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Compute gammas for decoder 1.
*******************************************************************************/
int Compute_Gamma_Dec1(inx,time,ip_array,p_array)
int inx,time,ip_array[],p_array[];
{
 int cnt,re_bit,im_bit;
 double re_sym,im_sym,re_sum,im_sum,re_err,im_err,x,re_pred,im_pred;
 re_bit=ip_array[ZERO];
 im_bit=p_array[ZERO];
 re_sum=Bit_Map[re_bit]-Re_Sym0[time];
 im_sum=Bit_Map[im_bit]-Im_Sym0[time];
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  re_bit=ip_array[cnt];
  im_bit=p_array[cnt];
  re_sym=Bit_Map[re_bit];
  im_sym=Bit_Map[im_bit];
  re_err=re_sym-Re_Sym0[time-cnt];
  im_err=im_sym-Im_Sym0[time-cnt];
  re_pred=Re_Pred_Coeff[PRED_MEM_SIZE][cnt];
  im_pred=Im_Pred_Coeff[PRED_MEM_SIZE][cnt];
  re_sum=re_sum+re_err*re_pred-im_err*im_pred;
  im_sum=im_sum+im_err*re_pred+re_err*im_pred;
 }
 x=re_sum*re_sum+im_sum*im_sum;
 x= -x*Inv_Twice_Pred_Var;
 Gamma_Dec1[time][inx]=exp(x);
// printf("time:%d inx:%d x:%lf g:%lf\n",time,inx,x,Gamma_Dec1[time][inx]);
 return(0);
}
/******************************************************************************/
/*                        Compute gammas for decoder 2.
*******************************************************************************/
int Compute_Gamma_Dec2(inx,time,ip_array,p_array)
int inx,time,ip_array[],p_array[];
{
 int cnt,re_bit,im_bit;
 double re_sym,im_sym,re_sum,im_sum,re_err,im_err,x,re_pred,im_pred;
 re_bit=ip_array[ZERO];
 im_bit=p_array[ZERO];
 re_sum=Bit_Map[re_bit]-Re_Sym1[time];
 im_sum=Bit_Map[im_bit]-Im_Sym1[time];
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  re_bit=ip_array[cnt];
  im_bit=p_array[cnt];
  re_sym=Bit_Map[re_bit];
  im_sym=Bit_Map[im_bit];
  re_err=re_sym-Re_Sym1[time-cnt];
  im_err=im_sym-Im_Sym1[time-cnt];
  re_pred=Re_Pred_Coeff[PRED_MEM_SIZE][cnt];
  im_pred=Im_Pred_Coeff[PRED_MEM_SIZE][cnt];
  re_sum=re_sum+re_err*re_pred-im_err*im_pred;
  im_sum=im_sum+im_err*re_pred+re_err*im_pred;
 }
 x=re_sum*re_sum+im_sum*im_sum;
 x= -x*Inv_Twice_Pred_Var;
 Gamma_Dec2[time][inx]=exp(x);
// printf("time:%d bit:%d x:%lf ge1:%lf\n",time,inx,x,Gamma_E_P1[time][inx]);
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec1_Process();
  if(Iter < (MAX_ITER-ONE))
   Dec2_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  else
   detected_bit=ONE;
  if(detected_bit != Bits[time])
  {
   Ber++;
// printf("time:%d prob 0:%lf prob 1:%lf\n",time,Prob[time][1],Prob[time][0]);
  }
 }
 return(0);
}
/******************************************************************************/
