/******************************************************************************/
#include "tc5ctype.h"
#include "tc5cext.h"
/******************************************************************************/
int Pre_Processor()
{
 Sim_Init();
 Open_Files();
 Get_Channel();
 Normalize_Channel_Energy();
 Get_Sym_Map();
 Get_Noise_Variance();
 Get_Iv_Map();
 Get_Enc_Taps();
 Get_Trellis();
 Get_Super_Trellis();
 Gamma_Mem_Alloc();
 Ck_Var();
 return(0);
}
/******************************************************************************/
/*                                Open files.
*******************************************************************************/
int Open_Files()
{
 fp1=fopen("tc5cdata/eqerr.dat","w");
 fp2=fopen("tc5cdata/chan.dat","r");
 fp3=fopen("tc5cdata/normchan.dat","w");
 fp4=fopen("tc5cdata/normeq.dat","w");
 if(fp2 == NULL)
  Error_Mes(14);
 return(0);
}
/******************************************************************************/
/*                       Get the channel coefficients.
*******************************************************************************/
int Get_Channel()
{
 int cnt;
 double re_tap,im_tap;
 cnt=ZERO;
 while(fscanf(fp2,"%lf %lf",&im_tap,&re_tap) != EOF)
 {
  if(cnt == CHAN_SIZE)
   Error_Mes(12);
  Re_Ch[cnt]=re_tap;
  Im_Ch[cnt]=im_tap;
  cnt++;
 }
 if(cnt != CHAN_SIZE)
  Error_Mes(13);
 return(0);
}
/******************************************************************************/
/*                  Normalize the channel to unit energy.
*******************************************************************************/
int Normalize_Channel_Energy()
{
 int cnt;
 double energy;
 energy=0.0;
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
  energy=energy+Re_Ch[cnt]*Re_Ch[cnt]+Im_Ch[cnt]*Im_Ch[cnt];
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
 {
  Re_Ch[cnt]=Re_Ch[cnt]/sqrt(energy);
  Im_Ch[cnt]=Im_Ch[cnt]/sqrt(energy);
  fprintf(fp3,"%lf %lf\n",Im_Ch[cnt],Re_Ch[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*                      Compute the noise variance.
*******************************************************************************/
int Get_Noise_Variance()
{
 int cnt;
 double ec;
 ec=0.0;
 for(cnt=ZERO;cnt<CONSTELL_SIZE;cnt++)
  ec=ec+Re_Map[cnt]*Re_Map[cnt]+Im_Map[cnt]*Im_Map[cnt];
 ec=ec/(double)(CONSTELL_SIZE);
 if(SNR > MAX_SNR)
  N_Var=0.0;
 else
  N_Var=ec/(2.0*pow(10.0,0.1*SNR));
 printf("\t Noise variance is %lf\n\n",N_Var);
 return(0);
}
/******************************************************************************/
/*                           Get encoder 1 taps.
*******************************************************************************/
int Get_Enc_Taps()
{
 Get_Enc_Numer_Taps();
 Get_Enc_Denom_Taps();
 return(0);
}
/******************************************************************************/
/*                      Get encoder 1 numerator taps.
*******************************************************************************/
int Get_Enc_Numer_Taps()
{
 Enc_N[0]=1;
 Enc_N[1]=0;
 Enc_N[2]=1;
 Enc_N[3]=1;
 Enc_N[4]=1;
 return(0);
}
/******************************************************************************/
/*                      Get encoder 1 denominator taps.
*******************************************************************************/
int Get_Enc_Denom_Taps()
{
 Enc_D[0]=1;
 Enc_D[1]=1;
 Enc_D[2]=0;
 Enc_D[3]=0;
 Enc_D[4]=1;
 return(0);
}
/******************************************************************************/
