/******************************************************************************/
#include "tc6ctype.h"
#include "tc6cext.h"
/******************************************************************************/
int Eq_Training(cnt)
int cnt;
{
 Get_Eq_Err(cnt);
 if(Frame_Cnt < (TRN_FRAMES-ONE))
  Update_Eq();
 if((Frame_Cnt==(TRN_FRAMES-ONE))&&(cnt>=COLLECT_START)&&(cnt<COLLECT_END))
  Collect_Data(cnt);
 return(0);
}
/******************************************************************************/
/*                         Get equalizer error.
*******************************************************************************/
int Get_Eq_Err(cnt)
int cnt;
{
 int time;
 double err_sq;
 time=(cnt-SYS_DELAY)/TWO;
 if(time >= FRAME_SIZE)
  Error_Mes(19);
 if(cnt < SYS_DELAY)
  Get_Buffer_Err();
 else if(((cnt%TWO) == ZERO) && (cnt >= SYS_DELAY))
  Get_Sym0_Err(time);
 else if(((cnt%TWO) == ONE) && (cnt >= SYS_DELAY))
  Get_Sym1_Err(time);
 err_sq=Re_Eq_Err*Re_Eq_Err+Im_Eq_Err*Im_Eq_Err;
 if(Frame_Cnt == TRN_FRAMES-ONE)
  fprintf(fp1,"%lf\n",err_sq);
 return(0);
}
/******************************************************************************/
/*                   Get the error from the buffer symbols.
*******************************************************************************/
int Get_Buffer_Err()
{
 Re_Eq_Err=  1.0-Re_Eq_Op;
 Im_Eq_Err= -1.0-Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                 Get the error from 1st encoder symbols.
*******************************************************************************/
int Get_Sym0_Err(time)
int time;
{
 Re_Eq_Err=Re_Sym0[time]-Re_Eq_Op;
 Im_Eq_Err=Im_Sym0[time]-Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                 Get the error from 2nd encoder symbols.
*******************************************************************************/
int Get_Sym1_Err(time)
int time;
{
 Re_Eq_Err=Re_Sym1[time]-Re_Eq_Op;
 Im_Eq_Err=Im_Sym1[time]-Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                Update the equalizer coefficients.
*******************************************************************************/
int Update_Eq()
{
 int cnt;
 Ip_Pow=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
  Ip_Pow=Ip_Pow+Re_Eq_Reg[cnt]*Re_Eq_Reg[cnt]+Im_Eq_Reg[cnt]*Im_Eq_Reg[cnt];
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  Re_Grad=Re_Eq_Err*Re_Eq_Reg[cnt]+Im_Eq_Err*Im_Eq_Reg[cnt];
  Im_Grad=Im_Eq_Err*Re_Eq_Reg[cnt]-Re_Eq_Err*Im_Eq_Reg[cnt];
  Re_Eq[cnt]=Re_Eq[cnt]+Step*Re_Grad/Ip_Pow;
  Im_Eq[cnt]=Im_Eq[cnt]+Step*Im_Grad/Ip_Pow;
 }
 return(0);
}
/******************************************************************************/
/*         Collect the data for updating the predictor coefficients.
*******************************************************************************/
int Collect_Data(cnt)
int cnt;
{
 if((cnt%TWO) == ZERO)
 {
  if(Err_Reg_Inx >= DATA_SIZE)
   Error_Mes(22);
  Re_Err_Reg[Err_Reg_Inx]=Re_Eq_Err;
  Im_Err_Reg[Err_Reg_Inx]=Im_Eq_Err;
  Err_Reg_Inx++;
  if(Err_Reg_Inx == DATA_SIZE)
  {
   Get_Eq_Err_Variance();
   Print_Norm_Eq();
  }
 }
 return(0);
}
/******************************************************************************/
/*                   Get the equalizer error variance.
*******************************************************************************/
int Get_Eq_Err_Variance()
{
 int cnt;
 double energy;
 energy=0.0;
 for(cnt=ZERO;cnt<DATA_SIZE;cnt++)
 {
  energy=energy+Re_Err_Reg[cnt]*Re_Err_Reg[cnt];
  energy=energy+Im_Err_Reg[cnt]*Im_Err_Reg[cnt];
 }
 energy=energy/(double)DATA_SIZE;
 Inv_Twice_Var=1.0/(2.0*energy);
 printf("\t Inv. of twice the noise variance:%lf\n\n",Inv_Twice_Var);
 return(0);
}
/******************************************************************************/
/*               Print the normalized equalizer coefficients.
*******************************************************************************/
int Print_Norm_Eq()
{
 int cnt;
 double energy,fac;
 energy=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  energy=energy+Re_Eq[cnt]*Re_Eq[cnt];
  energy=energy+Im_Eq[cnt]*Im_Eq[cnt];
 }
 fac=1.0/sqrt(energy);
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
  fprintf(fp4,"%lf %lf\n",fac*Im_Eq[cnt],fac*Re_Eq[cnt]);
 return(0);
}
/******************************************************************************/
