/******************************************************************************/
#include "tc7ctype.h"
#include "tc7cext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Get_Frame();
  if(Frame_Cnt != ZERO)
   Step=0.05;
  ISI_Procedures();
  if(Frame_Cnt >= TRN_FRAMES)
   Receive();
 }
 return(0);
}
/******************************************************************************/
/*                             ISI procedures.
*******************************************************************************/
int ISI_Procedures()
{
 int cnt;
 for(cnt=ZERO;cnt<(TWO*FRAME_SIZE+SYS_DELAY);cnt++)
 {
  Get_Channel_Op(cnt);
  Add_Noise();
  Get_Eq_Op();
  if(Frame_Cnt < TRN_FRAMES)
   Eq_Training(cnt);
  else if(cnt >= SYS_DELAY)
   Get_Noisy_Symbols(cnt);
 }
 return(0);
}
/******************************************************************************/
/*                          Get the channel output.
*******************************************************************************/
int Get_Channel_Op(cnt)
int cnt;
{
 Right_Shift_Reg(Re_Ch_Reg,Im_Ch_Reg,CHAN_SIZE);
 Shift_In_New_Ch_Inp(cnt);
 Compute_Ch_Op();
 return(0);
}
/******************************************************************************/
/*                        Shift in the new channel input.
*******************************************************************************/
int Shift_In_New_Ch_Inp(cnt)
int cnt;
{
 if(cnt<FRAME_SIZE)
 {
  Re_Ch_Reg[ZERO]=Re_Sym0[cnt];
  Im_Ch_Reg[ZERO]=Im_Sym0[cnt];
 }
 else if(cnt < (TWO*FRAME_SIZE))
 {
  Re_Ch_Reg[ZERO]=Re_Sym1[cnt-FRAME_SIZE];
  Im_Ch_Reg[ZERO]=Im_Sym1[cnt-FRAME_SIZE];
 }
 else
 {
  Re_Ch_Reg[ZERO]=  1.0;
  Im_Ch_Reg[ZERO]= -1.0;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the channel output.
*******************************************************************************/
int Compute_Ch_Op()
{
 int cnt;
 Re_Ch_Op=Im_Ch_Op=0.0;
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
 {
  Re_Ch_Op=Re_Ch_Op+Re_Ch[cnt]*Re_Ch_Reg[cnt];
  Re_Ch_Op=Re_Ch_Op-Im_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Re_Ch[cnt]*Im_Ch_Reg[cnt];
  Im_Ch_Op=Im_Ch_Op+Im_Ch[cnt]*Re_Ch_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                                Add noise.
*******************************************************************************/
int Add_Noise()
{
 int cnt;
 double erand48();
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(N_Var,N1_Val);
 Re_Ch_Op=Re_Ch_Op+N1_Val[GRAND_0];
 Im_Ch_Op=Im_Ch_Op+N1_Val[GRAND_1];
 return(0);
}
/******************************************************************************/
/*                          Get equalizer output.
*******************************************************************************/
int Get_Eq_Op()
{
 int cnt;
 Right_Shift_Reg(Re_Eq_Reg,Im_Eq_Reg,EQ_SIZE);
 Re_Eq_Reg[ZERO]=Re_Ch_Op;
 Im_Eq_Reg[ZERO]=Im_Ch_Op;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq[cnt]-Im_Eq_Reg[cnt]*Im_Eq[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq[cnt]+Im_Eq_Reg[cnt]*Re_Eq[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                 Get noisy symbols from equalizer output.
*******************************************************************************/
int Get_Noisy_Symbols(cnt)
int cnt;
{
 if(cnt < (SYS_DELAY+FRAME_SIZE))
 {
  Rx_Re_Sym0[cnt-SYS_DELAY]=Re_Eq_Op;
  Rx_Im_Sym0[cnt-SYS_DELAY]=Im_Eq_Op;
 }
 else
 {
  Rx_Re_Sym1[cnt-SYS_DELAY-FRAME_SIZE]=Re_Eq_Op;
  Rx_Im_Sym1[cnt-SYS_DELAY-FRAME_SIZE]=Im_Eq_Op;
 }
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Init_Gamma();
 Init_App();
 Bcjr();
 Get_Bit_Errors_Dec1();
 return(0);
}
/******************************************************************************/
/* Initialize the intrinsic and extrinsic values of gamma for decoder 1. Most
 * recent bit is at location ZERO.
*******************************************************************************/
int Init_Gamma()
{
 int time,inx,sum,ip_dec,p_dec;
 int ip_array[PRED_MEM_SIZE+1],p_array[PRED_MEM_SIZE+1];
 for(inx=ZERO;inx<Inx_To_VG_Tbl;inx++)
 {
  sum=Valid_Gamma_Inx[inx];
  ip_dec=sum%MOD1_NUM_INPUTS;
  p_dec=sum/MOD1_NUM_INPUTS;
  Decimal_to_Binary(ip_dec,ip_array,PRED_MEM_SIZE+ONE);
  Decimal_to_Binary(p_dec,p_array,PRED_MEM_SIZE+ONE);
  for(time=PRED_MEM_SIZE;time<FRAME_SIZE;time++)
  {
   Compute_Gamma_Dec1(inx,time,ip_array,p_array);
   Compute_Gamma_Dec2(inx,time,ip_array,p_array);
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Compute gammas for decoder 1.
*******************************************************************************/
int Compute_Gamma_Dec1(inx,time,ip_array,p_array)
int inx,time,ip_array[],p_array[];
{
 int cnt,sym_num;
 double re_sym,im_sym,re_sum,im_sum,re_err,im_err,x,re_pred,im_pred;
 sym_num=TWO*p_array[ZERO]+ip_array[ZERO];
 re_sum=Re_Map[sym_num]-Rx_Re_Sym0[time];
 im_sum=Im_Map[sym_num]-Rx_Im_Sym0[time];
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  sym_num=TWO*p_array[cnt]+ip_array[cnt];
  re_sym=Re_Map[sym_num];
  im_sym=Im_Map[sym_num];
  re_err=re_sym-Rx_Re_Sym0[time-cnt];
  im_err=im_sym-Rx_Im_Sym0[time-cnt];
  re_pred=Re_Pred_Coeff[PRED_MEM_SIZE][cnt];
  im_pred=Im_Pred_Coeff[PRED_MEM_SIZE][cnt];
  re_sum=re_sum+re_err*re_pred-im_err*im_pred;
  im_sum=im_sum+im_err*re_pred+re_err*im_pred;
 }
 x=re_sum*re_sum+im_sum*im_sum;
 x= -x*Inv_Twice_Pred_Var;
 Gamma_Dec1[time][inx]=exp(x);
// printf("time:%d inx:%d x:%lf g:%lf\n",time,inx,x,Gamma_Dec1[time][inx]);
 return(0);
}
/******************************************************************************/
/*                        Compute gammas for decoder 2.
*******************************************************************************/
int Compute_Gamma_Dec2(inx,time,ip_array,p_array)
int inx,time,ip_array[],p_array[];
{
 int cnt,sym_num;
 double re_sym,im_sym,re_sum,im_sum,re_err,im_err,x,re_pred,im_pred;
 sym_num=TWO*p_array[ZERO]+ip_array[ZERO];
 re_sum=Re_Map[sym_num]-Rx_Re_Sym1[time];
 im_sum=Im_Map[sym_num]-Rx_Im_Sym1[time];
 for(cnt=ONE;cnt<PRED_MEM_SIZE+ONE;cnt++)
 {
  sym_num=TWO*p_array[cnt]+ip_array[cnt];
  re_sym=Re_Map[sym_num];
  im_sym=Im_Map[sym_num];
  re_err=re_sym-Rx_Re_Sym1[time-cnt];
  im_err=im_sym-Rx_Im_Sym1[time-cnt];
  re_pred=Re_Pred_Coeff[PRED_MEM_SIZE][cnt];
  im_pred=Im_Pred_Coeff[PRED_MEM_SIZE][cnt];
  re_sum=re_sum+re_err*re_pred-im_err*im_pred;
  im_sum=im_sum+im_err*re_pred+re_err*im_pred;
 }
 x=re_sum*re_sum+im_sum*im_sum;
 x= -x*Inv_Twice_Pred_Var;
 Gamma_Dec2[time][inx]=exp(x);
// printf("time:%d bit:%d x:%lf ge1:%lf\n",time,inx,x,Gamma_E_P1[time][inx]);
 return(0);
}
/******************************************************************************/
/*                Initialize the a posteriori probabilities.
*******************************************************************************/
int Init_App()
{
 int time,input;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  for(input=ZERO;input<NUM_INPUTS;input++)
   Prob[time][input]=0.5;
 }
 return(0);
}
/******************************************************************************/
/*                          The BCJR algorithm.
*******************************************************************************/
int Bcjr()
{
 for(Iter=ZERO;Iter<MAX_ITER;Iter++)
 {
  Dec1_Process();
  if(Iter < (MAX_ITER-ONE))
   Dec2_Process();
 }
 return(0);
}
/******************************************************************************/
/*    Finally compute the number of bit errors in the frame from decoder1.
*******************************************************************************/
int Get_Bit_Errors_Dec1()
{
 int time,detected_bit;
 for(time=ZERO;time<FRAME_SIZE;time++)
 {
  if(Prob[time][ZERO] > Prob[time][ONE])
   detected_bit=ZERO;
  else
   detected_bit=ONE;
  if(detected_bit != Bits[time])
  {
   Ber++;
// printf("time:%d prob 0:%lf prob 1:%lf\n",time,Prob[time][1],Prob[time][0]);
  }
 }
 return(0);
}
/******************************************************************************/
